#! /usr/bin/perl

#   	    	Merge PostScript files, expanding
#   	      "(file.ps) run" to in-line inclusion.

#   	         by John Walker -- November 2003
#   	    	    http://www.fourmilab.ch/
#   	       This program is in the public domain.

#   Note:  this program is intended to be used with PostScript
#   explicitly prepared for it.  It does not contain a general
#   PostScript parser, and will mess up unless each inclusion is
#   on a line by itself precisely as described in the title.
#   Inclusions may be recursive to any depth Perl function
#   recursion and available file handles permit.

    require 5;
    use strict;

    if ($#ARGV < 0) {
    	&usage();
    }
    my $stripinc = 1;
    
    if ($#ARGV > 0) {
    	if ($ARGV[0] eq '-l') {
	    $stripinc = 0;
	    shift(@ARGV);
	} else {
	    &usage();
	}
    }
    
    &copyfile($ARGV[0], 0);
    
    sub copyfile {
    	my ($file, $strip) = @_;
	my ($l, $fi);

#print("% * Copyfile opening $file with strip mode = $strip.\n");	
	open($fi, "<$file") || die("Cannot open input file $file");

	while ($l = <$fi>) {
    	    chomp($l);

#print(".. $l\n");
	    if ($l =~ m/\s*\(([\w\.]+)\)\s*run/) {
    		my $addfile = $1;
#print("% ++$addfile\n");
    	    	&copyfile($addfile, $stripinc);
	    } else {
	    	if ($strip) {
		    $l =~ s/%[^\)]*$//;
		    $l =~ s/\s+$//;
		    if ($l =~ m/^\s*$/) {
		    	next;
		    }
		}
		if ($l =~ m/^%!/) { 	# PostScript version lines always ditched
		    next;
		}
	    	print("$l\n");
	    }
	}
	close($fi);
    }
    
    sub usage {
    	print(STDERR "Usage: psmerge.pl [ -l ] psfile.ps\n");
	print(STDERR "  Options:  -l   Preserve comments and blank lines in inclusions\n");
	exit(2);
    }
